declare namespace $ { }
export = $;
/**
 * Return `unknown` when `A` and `B` are the same type. `never` otherwise.
 *
 * 	$mol_type_equals< unknown , any > & number // never
 * 	$mol_type_equals< never , never > & number // number
 */
declare type $mol_type_equals<A, B> = (<X>() => X extends A ? 1 : 2) extends (<X>() => X extends B ? 1 : 2) ? unknown : never;
/**
 * Asserts for equality of `Actual` and `Expected` types.
 * Don't use `never` as `Expected` - use `$mol_type_assert_never` instead.
 */
declare type $mol_type_assert<Actual, Expected extends $mol_type_equals<Actual, Expected>> = Actual;
/**
 * Asserts for `Actual` type is `never`.
 *
 * 	$mol_type_assert_never< $mol_type_equals< 1 , 2 > >
 */
declare type $mol_type_assert_never<Actual extends never> = Actual;
/**
 * Returns `Tuple` without first element.
 *
 * 	$mol_type_tail<[ 1 , 2 , 3 ]> // [ 2, 3 ]
 */
declare type $mol_type_tail<Tuple extends readonly any[]> = ((...tail: Tuple) => any) extends ((head: any, ...tail: infer Tail) => any) ? Tail : never;
/**
 * Returns first element of `Tuple`.
 *
 * 	$mol_type_tail<[ 1 , 2 , 3 ]> // 1
 */
declare type $mol_type_head<Tuple extends any[]> = Tuple['length'] extends 0 ? never : Tuple[0];
/**
 * Converts union of types to intersection of same types
 *
 * 	$mol_type_intersect< number | string > // number & string
 */
declare type $mol_type_intersect<Union> = (Union extends any ? (_: Union) => void : never) extends ((_: infer Intersection) => void) ? Intersection : never;
/**
 * Extracts keys from `Input` which values extends `Upper`, but not extends `Lower`.
 *
 * 	type MathConstants = $mol_type_keys_extract< Math , never , number > // "E" | "PI" ...
 */
declare type $mol_type_keys_extract<Input, Lower, Upper> = {
    [Field in keyof Input]: Input[Field] extends Lower ? never : Input[Field] extends Upper ? Field : never;
}[keyof Input];
/**
 * Exclude keys from `Input` which values extends `Upper`, but not extends `Lower`.
 *
 * 	type MathConstants = $mol_type_keys_exclude< Math , never , Function > // "E" | "PI" ...
 */
declare type $mol_type_keys_exclude<Input, Lower, Upper> = Exclude<keyof Input, $mol_type_keys_extract<Input, Lower, Upper>>;
/**
 * Reqursive converts intersection of records to record of intersections
 *
 * 	// { a : 1 & 2 }
 * 	$mol_type_merge< { a : 1 } & { b : 2 } >
 */
declare type $mol_type_merge<Intersection> = keyof Intersection extends string ? {
    [Key in keyof Intersection]: $mol_type_merge<Intersection[Key]>;
} : Intersection;
/**
 * Omit keys from `Input` which values extends `Upper`, but not `Lower`.
 *
 * 	type MathConstants = $mol_type_omit< Math , never , Function > // { E , PI , ... }
 */
declare type $mol_type_omit<Input, Lower, Upper> = Pick<Input, $mol_type_keys_exclude<Input, Lower, Upper>>;
/**
 * Recursive `Partial`.
 *
 * 	let props : $mol_type_partial_deep< HTMLElement > = { style : { display : 'block' } }
 */
declare type $mol_type_partial_deep<Val> = {
    [field in keyof Val]?: $mol_type_partial_deep<Val[field]>;
};
/**
 * Picks keys from `Input` which values extends `Upper`, but not `Lower`.
 *
 * 	type MathConstants = $mol_type_pick< Math , never , number > // { E , PI , ... }
 */
declare type $mol_type_pick<Input, Lower, Upper> = Pick<Input, $mol_type_keys_extract<Input, Lower, Upper>>;
/**
 * Returns `Tuple` unshifted by `Item`.
 *
 * 	$mol_type_tail< 1 , [ 2 , 3 ] > // [ 1, 2 , 3 ]
 */
declare type $mol_type_prepend<Item extends any, Tuple extends readonly any[]> = ((head: Item, ...args: Tuple) => any) extends ((...args: infer Result) => any) ? Result : Tuple;
